#include "power_supply.h"

void AppInit(void){
	
	//Init ADC converter    
	AdcInit();

	//Init PWM with init value 0
   PwmInit();

	//Init Timer0 for 1ms Interrupt
	T0Init();

	//Config DDR & PORT reg. for buttons	
	ButtonsInit();
	
	//LCD Init
	lcd_init(LCD_DISP_ON);
	
	//Clear LCD
	lcd_clrscr();
	
	//Delay 500mS
	_delay_ms(500);

	//Set adc target current
	target_adc = 20;
	
	//Update SwUp variable
	switch_up = SwUp();
		
	//Update SwDown variable
	switch_down = SwDown();
	
	//Enable Global Interrupts
	sei();
}

//------------------LCD Refresh--------------------------------

void LcdRun(void){

	//Timeout for LCD refresh or forced refresh?
	if( (t_lcd > 100) || force_LCD_refresh){
		
		//Reset forced LCD refresh flag		
		force_LCD_refresh = FALSE;

		//Reset timer for new refresh
		t_lcd = 0;

		//auto variables for LCD refresh
		char buffer[4],buffer1[4];

		//Disable global interrupts
		cli();	

		//convert interger into string 
		itoa( target_adc , buffer, 10);
		itoa( Adc_value , buffer1, 10);

		//Enable Global interrupts
		sei();	
		
		//Clear LCD
		lcd_clrscr();
		
		//Refresh
		lcd_gotoxy(0,0); lcd_puts("ADC=\n");
		lcd_gotoxy(4,0); lcd_puts(buffer1);
		lcd_gotoxy(0,1); lcd_puts("TGT=\n");
		lcd_gotoxy(4,1); lcd_puts(buffer);
			
	}
}

void ButtonsRun(void){

//----------B U T T O N S   N O R M A L   A D V A N C E -------	

//----------B U T T O N   U P -----------

	//switc_up same like variable?
	if(switch_up == SwUp()){
		
		//reset debounce timer
		t_switch_up = 0;	
	}
	//debounce timeout?
	else if((t_switch_up > 50) && switch_down){

		//reset debounce timer
		t_switch_up = 0;

		//Update switch variable
		switch_up = SwUp();
		
		//switch_up active?		
		if(!switch_up){
			
			//Target adc < MAX ADC value
			if(target_adc < 1023){
				
				//Increment target_adc
				target_adc++;

				//force LCD to refresh
				force_LCD_refresh = TRUE;
			}
		}
	}

//----------B U T T O N   D O W N -------

	//switc_down same like variable?
	if(switch_down == SwDown()){
		
		//reset debounce timer
		t_switch_down = 0;
	}
	//debounce timeout?
	else if((t_switch_down > 50) && switch_up){

		//reset debounce timer
		t_switch_down = 0;

		//Update switch variable
		switch_down = SwDown();
		
		//switch_down active?		
		if(!switch_down){
			
			//target adc > 0?
			if(target_adc){

				//Decrement target_adc
				target_adc--;

				//force LCD to refresh
				force_LCD_refresh = TRUE;
			}
		}
	}

//----------B U T T O N S   A U T O   A D V A N C E -----------	

	//Active UP key
	if(switch_up == 0){
		
		//Time for advance target_adc?
		if(t_switch_auto > advance_tick){
			
			//Reset timer for new advance
			t_switch_auto = 0;

			//Speed up auto advance
			advance_tick -= 50;

			//Check max speed
			if(advance_tick < 70){

				//Set max advance speed
				advance_tick = 70;
			}

			//Target adc < MAX ADC value
			if(target_adc < 1023){

				//Increment target current
				target_adc++;

				//force LCD to refresh
				force_LCD_refresh = TRUE;
			}
		}
	}
	//Active Down key
	else if(switch_down == 0){
		
		//Time for advance target_adc?
		if(t_switch_auto > advance_tick){
			
			//Reset timer for new advance
			t_switch_auto = 0;

			//Speed up auto advance
			advance_tick -= 50;

			//Check max speed
			if(advance_tick < 70){

				//Set max advance speed
				advance_tick = 70;
			}

			//target adc > 0?
			if(target_adc){

				//Decrement target current
				target_adc--;

				//force LCD to refresh
				force_LCD_refresh = TRUE;
			}
		}
	}
	else{
			
		//Reset timer for auto advance
		t_switch_auto = 0;
		
		//Set advance tick to start value 500mS
		advance_tick = 500;
	}	
}
